#ifndef __D3D_EXPORT_H
#define __D3D_EXPORT_H

#include <objbase.h>
#include <ddraw.h>
#include <d3d.h>
#include <d3drm.h>
#include <d3drmdef.h>

#undef RELEASE
#ifdef __cplusplus
#define RELEASE(x) if (x != NULL) {x->Release(); x = NULL;}
#else
#define RELEASE(x) if (x != NULL) {x->lpVtbl->Release(x); x = NULL;}
#endif

#define SUPPORTED_DXVERSION 6
//------------------------------------------------------------
//Flags for purpose off call Draw2dCallback(HDC,type) and Common2dCallback(GraphInfo,type)
//don't call callback
#define _2D_CANCEL                         0
//draw before render
#define _2D_BEFORE                         1
//draw before render left SI
#define _2D_BEFORE_LEFT_SI                 2
//draw before render right SI
#define _2D_BEFORE_RIGHT_SI                3
//draw after render
#define _2D_AFTER                          4
//draw after render SI
#define _2D_AFTER_SI                       5
//draw after render left
#define _2D_AFTER_LEFT_SI                  6
//draw after render right
#define _2D_AFTER_RIGHT_SI                 7
//draw after render for copy
#define _2D_AFTER_COPY                     8
//draw after render SI for copy
#define _2D_AFTER_SI_COPY                  9

//-------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//Struct SceneInfo(contain only pointer)
struct SceneInfo
{
    LPDIRECT3DRMFRAME3   lpTopFrame;        	// scene frame
    LPDIRECT3DRMLIGHT    lpAmbient;      		// scene ambient light
};


//Struct GraphInfo(contain only pointer)
struct GraphInfo
{
    LPDIRECT3DRM3        lpD3DRM3;        // Direct3DRM2 Interface
    LPDIRECT3DRM         lpD3DRM;         // Direct3DRM Interface

    LPDIRECTDRAW4        lpDD2;           // DirectDraw2 Interface
    LPDIRECT3D3          lpD3D;           // Direct3D2 Interface
    LPDIRECT3DRMDEVICE3  lpD3DRMDevice;   // Direct3DRM2 device
    LPDIRECT3DRMVIEWPORT2 lpD3DRMViewport; // Direct3DRM2 Viewport
    LPDIRECT3DRMFRAME3   lpCamera;        // Camera frame
    LPDIRECT3DRMFRAME3   lpCameraLeft;    // Left Camera frame
    LPDIRECT3DRMFRAME3   lpCameraRight;   // Right Camera frame
    LPDIRECT3DDEVICE3    lpd3dDevice;	  // D3D Device
    LPDIRECT3DVIEWPORT3  lpd3dViewport;	  // D3D Viewport
    LPDIRECTDRAWSURFACE4 lpddsPrimary;    // Primary Surface
    LPDIRECTDRAWSURFACE4 lpddsRender;     // Rendering surface
    LPDIRECTDRAWSURFACE4 lpddsZBuff;      // Z-Buffer surface
    LPDIRECTDRAWCLIPPER  lpDDClipper;     // Clipper object

//    HWND hwnd;                            // output window
//    HDC  hdc;                             // output device context
};

//------forward declaration--------
interface I_D3DWindow;
typedef I_D3DWindow* LPI_D3DWindow;
interface I_Scene;
typedef I_Scene* LPI_Scene;
//---------------------------------

interface I_D3DWindow
{
  virtual HRESULT STDMETHODCALLTYPE I_AttachScene(LPI_Scene lpScene)=0;
  virtual HRESULT STDMETHODCALLTYPE I_DetachScene(void)=0;
  virtual void STDMETHODCALLTYPE I_GetGraphInfo(GraphInfo&)=0;
  virtual BOOL STDMETHODCALLTYPE I_Draw(void)=0;
  virtual HRESULT STDMETHODCALLTYPE I_CombineInterlacedFrame(void)=0;

  virtual BOOL STDMETHODCALLTYPE I_isValid(void)=0;
  virtual HRESULT STDMETHODCALLTYPE I_GetSurfaceRect(RECT&)=0;
  virtual BOOL STDMETHODCALLTYPE I_NeedDXUpdateCall(void)=0;

  virtual void STDMETHODCALLTYPE I_SetStereoBaseAndDistanse(float base,float distanse)=0;
  virtual HRESULT STDMETHODCALLTYPE I_StartSI(void)=0;
  virtual HRESULT STDMETHODCALLTYPE I_StopSI(void)=0;

  virtual float STDMETHODCALLTYPE I_Pick(int x, int y,LPDIRECT3DRMFRAME3* lppick_frame)=0;
};

interface I_Scene
{
  virtual HRESULT STDMETHODCALLTYPE I_Init(LPI_D3DWindow lpd3dWindow)=0;
  virtual HRESULT STDMETHODCALLTYPE I_Fini(void)=0;
  virtual HRESULT STDMETHODCALLTYPE I_Attach(void)=0;
  virtual HRESULT STDMETHODCALLTYPE I_Detach(void)=0;
  virtual HRESULT STDMETHODCALLTYPE I_Render(void)=0;
  virtual HRESULT STDMETHODCALLTYPE I_RenderSI(void)=0;
  virtual HRESULT STDMETHODCALLTYPE I_SceneSetDraw2dCallback(BOOL (CALLBACK *fn)(HDC,D3DRECT,LPDDSURFACEDESC2,int))=0;
  virtual HRESULT STDMETHODCALLTYPE I_SceneSetCommon2dCallback(bool CALLBACK (__closure *fn)(GraphInfo,D3DRECT,int))=0;

  virtual HRESULT STDMETHODCALLTYPE I_GetSceneInfo(SceneInfo&)=0;
};

//Export functions
  __declspec(dllexport) BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/);
  __declspec(dllexport) BOOL WINAPI D3DWindowCreateD3DRM(void);
  __declspec(dllexport) BOOL WINAPI D3DWindowDestroyD3DRM(void);
//D3DWindow function
  __declspec(dllexport) LPI_D3DWindow WINAPI D3DWindowCreate(void);
  __declspec(dllexport) BOOL WINAPI D3DWindowSetMouseCallback(BOOL (CALLBACK *fn)(HWND, UINT, WPARAM, LPARAM));
  __declspec(dllexport) BOOL WINAPI D3DWindowSetKeyboardCallback(BOOL (CALLBACK *fn)(HWND, UINT, WPARAM, LPARAM));
  __declspec(dllexport) INT  WINAPI D3DWindowRun(void/*HWND hwnd*/);
  __declspec(dllexport) BOOL WINAPI D3DWindowGetGraphInfo(GraphInfo&);
//RMWindow function
  __declspec(dllexport) LPI_D3DWindow WINAPI RMWindowCreate(int cmdshow=SW_SHOWNORMAL);
  __declspec(dllexport) BOOL WINAPI RMWindowSetMouseCallback(BOOL (CALLBACK *fn)(HWND, UINT, WPARAM, LPARAM));
  __declspec(dllexport) BOOL WINAPI RMWindowSetKeyboardCallback(BOOL (CALLBACK *fn)(HWND, UINT, WPARAM, LPARAM));
  __declspec(dllexport) void WINAPI RMWindowRun(void);
  __declspec(dllexport) void WINAPI RMWindowOnIdle(void);
//OffScreenDevice function
  __declspec(dllexport) LPI_D3DWindow WINAPI OffScreenDeviceCreate(HWND hwnd,int w,int h,D3DCOLORMODEL model,BOOL bSoftwareOnly);
  __declspec(dllexport) void WINAPI OffScreenDeviceDestroy(LPI_D3DWindow OffScreenDevice);
//Scene function
  __declspec(dllexport) BOOL WINAPI SceneSetCreateSceneCallback(BOOL (CALLBACK *fn)(LPDIRECT3DRM3 lpD3DRM,LPDIRECT3DRMFRAME3 scene));
  __declspec(dllexport) LPI_Scene WINAPI CreateScene(void);

#ifdef __cplusplus
}
#endif

#endif // End __D3D_EXPORT_H
