//---------------------------------------------------------------------------
#ifndef ImmFileH
#define ImmFileH

#include <vcl.h>
#include <d3d.h>
#include <d3drm.h>
#include <vector.h>

#include "sc_tex.h"
#include "triarray.h"
#include "app.h"
//#include "sc_immscene.h"

#define IMM_ID 1234567
#define IMM_VERS 100
#define IMM_TXTRSIZE 256
enum { IMM_INDICESPRMT, IMM_STRIP3PRMT };
//---------------------------------------------------------------------------
struct  PARAMS
{
   D3DTexture	*Texture;
   int          VertexCount, IndicesCount;
   D3DVERTEX    *Vertex;
   WORD	        *VertexIndices;
};

struct  PACKAGE Block
{
  Surface *surf;
  FList  *faces;
  RECT rect;
  Block();
  ~Block();
};

struct  HImmBlock
{
  int   ColumnNum, RowNum;
  int   NumVertexs;
  int   NumFaces;
  D3DVERTEX *vrtx;
  unsigned short *indc;
  D3DVECTOR min, max;
  RECT  TextureBlock;
  D3DVECTOR paramsz;               //  -
  int   reserv[20];
  HImmBlock()  { ColumnNum=RowNum=NumVertexs=NumFaces=0; vrtx=NULL,indc=NULL; }
};

struct  HLayer
{
  int   LrNum;
  int   Columns,  Rows;
  char  TextureFile[128];
  float eps;
  HImmBlock *blks;
  int   reserv[20];
  HLayer() { LrNum=Columns=Rows=eps=0; TextureFile[0]=0; blks=NULL; }
};

struct  HImmFile
{
  int   ID;
  int   Vers;
  char  Name[32];
  char  Descr[96];
  int   NumLayers;
  HLayer *lrs;
  int   TextureSize;
  D3DVECTOR min, max;
  int   type;
  int   reserv[19];
  HImmFile() {ID=Vers=Name[0]=Descr[0]=NumLayers=TextureSize=0; lrs=NULL;  }
};

class PACKAGE SurfFile {
  TStereoControl *d3panel;
  HImmFile fhdr;

  void SetDefaultBmpDir(AnsiString dir);
  char *SetLayerTexture(int Layer, char *txtr);        // read method
  char *GetLayerTexture(int Layer);                    // write method
  int   GetLayerNumRows(int layer);
  int   GetLayerNumCols(int Layer);
  int   GetTxtrSize();
  int   GetType();
  float SetLayerEps(int layer, float eps);
  float GetLayerEps(int layer);
  RECT  GetRect(int layer, int ind) { return fhdr.lrs[layer].blks[ind].TextureBlock; }
  D3DMATERIAL FMaterial;
public:
   __property  float eps[int layer] = { read=GetLayerEps, write=SetLayerEps};
   __property  char *textures[int layer] = { read=GetLayerTexture, write=SetLayerTexture};
   __property  int NumRows[int layer] = { read=GetLayerNumRows};
   __property  int NumCols[int layer] = { read=GetLayerNumCols};
   __property  int NumLayers = { read=fhdr.NumLayers};
   __property  D3DVECTOR min = { read=fhdr.min, write=fhdr.min};
   __property  D3DVECTOR max = { read=fhdr.max, write=fhdr.max};
   __property  int TextureSize = { read=GetTxtrSize};
   __property  int Type = { read=GetType};
   __property  RECT SourceTexture[int layer][int ind] = { read=GetRect };
   __property int VisibleLayer  = { read = FVisibleLayer, write = SetVisibleLayer };
   __property int BlocksCount  = { read = GetBlocksCount };
   __property AnsiString DefaultBmpDir  = { read = FDefaultBmpDir, write = SetDefaultBmpDir };
   __property  D3DMATERIAL Material = { read=FMaterial, write=FMaterial};
   vector<TD3TriArray*> Blocks;

  SurfFile(char *file=NULL, TStereoControl * panel=NULL );
  SurfFile(int Layers, int HBlocks, int VBlocks, int type, TStereoControl * panel=NULL );
  ~SurfFile();

  bool SaveToFile( char *file);
  bool SaveToXFile( char *file);
  bool SaveToSrfFile( char *file);
  bool LoadFromFile(char *file);
  bool GetBlock(int Layer, int Col, int Row,  Block &blk);
  bool SaveBlock(int Layer, int Col, int Row, Block &blk);
  void ClearBlock(Block &blk);
  bool SaveBlock(int Layer, int Col, int Row, Surface &surf, FList  &faces, RECT &rect,bool build_normals=false);
  bool GetBlock(int Layer, int Col, int Row,  Surface &surf, FList  &faces, RECT &rect);
  bool GetBlock(int Layer, int Col, int Row, PARAMS &prms, RECT &rect);
  bool GetBlock(int Layer, int ind, PARAMS &prms, RECT &rect);
  void GetBox( D3DRMBOX * lpD3DRMBox  );
  void Normalize(void);
  void Scale(float dx, float dy, float dz);
  TStereoControl * __fastcall AttachToPanel(TStereoControl * panel);
private:
    int FVisibleLayer;
    AnsiString FDefaultBmpDir;
    void __fastcall SetVisibleLayer(int value);
    int __fastcall GetBlocksCount();
    AnsiString & __fastcall GetBmpFile(AnsiString src);
};
#endif

