//---------------------------------------------------------------------------
#ifndef Panel3DH
#define Panel3DH
#define D3D_OVERLOADS
#define DIRECTDRAW_VERSION 0x0600
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>
#include <vector.h>
#include <syncobjs.hpp>
#include <DsgnIntf.hpp>
#pragma hdrstop

#include "sc_rmscene.h"
#include "wn_rmwin.h"
#include "service3d.h"
#include "export.h"
#include "objects.h"

#include "models.h"
#include "objects.h"
#include "camera.h"
#include "light.h"
#include "d3matrl.h"
#include "d3matrl.h"
#include "properties.h"
#include "common.h"
#include "stctrl.h"

#define DELETE(X)   if(X) { delete X; X=NULL;}
typedef enum { taLeftJustify, taRightJustify, taCenter,
		 taLeftTopJustify, taRightTopJustify,taLeftBottomJustify, taRightBottomJustify} TAlignmentEx;
//---------------------------------------------------------------------------
class PACKAGE TStereoControl : public TCustomControl
{
	 friend class        TD3CommonLight;
	 friend class        TD3CameraProp;
	 friend class        TD3Visual;
	 friend class        TD3Engine;
	 friend class        TStereo;
private:
	TD3Visual           *Fd3Selected;
	TD3Fog              *FFog;
	TD3Engine           *FD3;
	TStereo             *FStereo;
	TD3CommonLight      *FLight;
	TD3CameraProp       *FCamera;
	TNotifyEvent        FOnLoaded;
	TNotifyEvent 		  FOnNeedReload;
	TCriticalSection    *CriticalSection;
	TD3Camera           *FMainCamera;
	bool                EndLoad;
	HPALETTE            FPalette;
	TCursor             FCursor2;
	TTimer              *Timer;
	Graphics::TBitmap	  *FIconLeft, *FIconRight;
	bool					  FShowIcon, FAutoUpdate, FStereoCursor;
	TAlignment			  FAlignment;

	TColor  __fastcall  GetColor(void);
	void    __fastcall  SetColor       (TColor value);
	void    __fastcall  Setd3Selected  (TD3Visual *value);
	void    __fastcall  SetFog         (TD3Fog *value);
	void    __fastcall  SetD3Drv       (DRV value);
	void    __fastcall  SetD3          (TD3Engine *value);
	void    __fastcall  SetLight       (TD3CommonLight *value);
	void    __fastcall  SetCamera      (TD3CameraProp *value);
	void    __fastcall  SetPalette     (HPALETTE value);
	void    __fastcall  SetIconLeft    (Graphics::TBitmap *value);
	void    __fastcall  SetIconRight   (Graphics::TBitmap *value);
	void    __fastcall  SetShowIcon	  (bool value);
	DYNAMIC 	HPALETTE   __fastcall     	GetPalette     (void);
	DYNAMIC 	void       __fastcall    	MouseMove(Classes::TShiftState Shift, int X, int Y);
	DYNAMIC 	void 		  __fastcall 	   MouseDown(TMouseButton Button, Classes::TShiftState Shift, int X, int Y);
protected:
	virtual void  __fastcall   Sync_2D_3D       (void);
	virtual void  __fastcall   Loaded           (void);
	virtual bool  CALLBACK     DefaultCommon2DHandler(GraphInfo grInfo,D3DRECT ClipRect,int type);
	virtual void  __fastcall   ShowObjCaptures  (LPDIRECTDRAWSURFACE4 lpddsD3);
	virtual void  __fastcall   OnTimer          (System::TObject* Sender);
	virtual void  __fastcall   SetCursor        (TCursor Value);
public:
	__fastcall    TStereoControl(TComponent* Owner);
	__fastcall    ~TStereoControl();

	RMWindow      *OffScrDev;
	RMScene       *D3DRMScene;

	GraphInfo     Interfaces;
	TD3Visual     *Scene;
	TD3Light      *FAmbientLight, *FDirectionalLight;
	TD3Models     Models;
	TD3Visuals    Visuals;
	TStringList   *Textures;
	D3DVECTOR     SkyScale,GroundScale;            // xxxScale -     
	TD3Panorama   *Sky, *Ground;                   //     
	TGlasses      *Glasses;

	 __property    Canvas;
	__property    TStereo        *Stereo     = { read = FStereo,     write = FStereo };
	__property    TD3Engine      *D3         = { read = FD3,         write = SetD3 };
	__property    TD3Visual      *d3Selected = { read = Fd3Selected, write = Setd3Selected };
	__property    TD3CommonLight *Light      = { read = FLight,      write = SetLight };
	__property    TD3CameraProp  *Camera     = { read = FCamera,     write = SetCamera };
	__property    TD3Fog         *Fog        = { read = FFog,        write = SetFog };
	__property    HPALETTE       Palette     = { read = GetPalette,  write = SetPalette,     nodefault};

	virtual       void           __fastcall     Repaint(void );
	virtual       TD3Visual*     __fastcall     GetObject(int x, int y);
	virtual       void           __fastcall     Update(void);
	virtual       void           __fastcall     Paint(void);

__published:
	__property Align;
//	__property Caption ;
//	__property Font ;
	__property ParentColor ;
	__property ParentShowHint ;
	__property PopupMenu ;
	__property ShowHint ;
	__property TabStop ;
	__property TabOrder ;
	__property Visible ;
	__property WheelAccumulator ;
	__property OnCanResize ;
	__property OnClick ;
	__property OnContextPopup ;
	__property OnDblClick ;
	__property OnDragDrop ;
	__property OnDragOver ;
	__property OnEndDrag ;
	__property OnEnter ;
	__property OnExit ;
	__property OnMouseDown ;
	__property OnMouseMove ;
	__property OnMouseUp ;
	__property OnMouseWheel;
	__property OnMouseWheelUp;
	__property OnMouseWheelDown;
	__property OnKeyDown;
	__property OnKeyUp;
	__property OnKeyPress;
	__property OnResize ;
	__property OnStartDrag ;

	__property TCursor      Cursor     = { read = FCursor2,    write = SetCursor};
	__property TColor		   Color = { read = GetColor, write=SetColor, default= clBlack };
	__property TAlignment	Alignment = { read = FAlignment, write=FAlignment, default=::taLeftJustify};
	__property Graphics::TBitmap	*IconLeft = { read = FIconLeft, write=SetIconLeft};
	__property Graphics::TBitmap  *IconRight = { read = FIconRight, write=SetIconRight};
	__property bool			ShowIcon = { read = FShowIcon, write=SetShowIcon, default= false};
	__property bool     		AutoUpdate={ read = FAutoUpdate, write = FAutoUpdate, default=true };
	__property bool     		StereoCursor={ read = FStereoCursor, write = FStereoCursor, default=true };

	__property TNotifyEvent OnLoaded    = { read = FOnLoaded,   write = FOnLoaded };
	__property TNotifyEvent OnNeedReload= { read = FOnNeedReload,  write = FOnNeedReload };
};
//---------------------------------------------------------------------------
#endif

