#ifndef __LIGHT__
#define __LIGHT__
/*
**-----------------------------------------------------------------------------
** Name:    Light.h
** Purpose: Various D3D IMM light functions
** Notes:
**
** Copyright (c) 1995 - 1997 by Microsoft, all rights reserved
**-----------------------------------------------------------------------------
*/

#define D3D_OVERLOADS
#include <d3d.h>
#include <d3dtypes.h>
#include <list>
#include <vcl.h>
#pragma hdrstop

#include "export.h"

class Light;  //forward declaration
typedef Light* LPLight;
typedef std::list<LPLight> LIST_Light;  //std::  instead of (using namespace std;)

/*
**-----------------------------------------------------------------------------
** Classes
**-----------------------------------------------------------------------------
*/

/*
**-----------------------------------------------------------------------------
**	Class:		Light
**	Purpose:
**-----------------------------------------------------------------------------
*/
class PACKAGE Light {

protected:
	static LIST_Light list;
	D3DLIGHT2			light;		// structure defining the light
	LPDIRECT3DLIGHT		lpD3DLight;	// object pointer for the light
	int					changed;
public:
	Light();
	virtual ~Light();
    void CreateLight(LPDIRECT3D2 lpD3D);

	HRESULT		AddToViewport(LPDIRECT3DVIEWPORT2 lpView);
	HRESULT		RemoveFromViewport(LPDIRECT3DVIEWPORT2 lpView);
	HRESULT		Set(void);

	void		SetColor(const D3DVECTOR& color);
	D3DVECTOR	GetColor(void) const;
	void		SetPosition(const D3DVECTOR& position);
	D3DVECTOR	GetPosition(void) const;
	void		SetDirection(const D3DVECTOR& direction);
	D3DVECTOR	GetDirection(void) const;
	void		SetAttenuation(const D3DVECTOR& attenuation);
	D3DVECTOR	GetAttenuation(void) const;
	void		SetRange(const float range);
	float		GetRange(void) const;
	void		SetFalloff(const float falloff);
	float		GetFalloff(void) const;
	void		SetUmbra(const float falloff);
	float		GetUmbra(void) const;
	void		SetPenumbra(const float falloff);
	float		GetPenumbra(void) const;
	void		SetFlags(const DWORD flags);
	DWORD		GetFlags(void) const;

	static LIST_Light& GetList() { return list; }
	static void AfterDeviceCreated(GraphInfo&);
	static void BeforeDeviceDestroyed(GraphInfo&);
protected:
	void Release(void);

}; // End Light


/*
**-----------------------------------------------------------------------------
**	Class:		PointLight
**	Purpose:
**-----------------------------------------------------------------------------
*/

class PointLight : public Light {
	public:
		PointLight(	const D3DVECTOR & color,
					const D3DVECTOR & position);
}; // End PointLight


/*
**-----------------------------------------------------------------------------
**	Class:		SpotLight
**	Purpose:
**-----------------------------------------------------------------------------
*/

class SpotLight : public Light {
	public:

	  SpotLight(const D3DVECTOR& color,
				const D3DVECTOR& position,
				const D3DVECTOR& direction,
				const float umbra_angle,
				const float penumbra_angle);
}; // End SpotLight



/*
**-----------------------------------------------------------------------------
**	Class:		DirectionalLight
**	Purpose:
**-----------------------------------------------------------------------------
*/

class DirectionalLight : public Light {
	public:
		DirectionalLight(const D3DVECTOR& color,
			             const D3DVECTOR& direction);
};



/*
**-----------------------------------------------------------------------------
**	Class:		ParallelPointLight
**	Purpose:
**-----------------------------------------------------------------------------
*/

class ParallelPointLight : public Light {
	public:
		ParallelPointLight(const D3DVECTOR& color,
			               const D3DVECTOR& position);
}; // End ParallelPointLight

/*
**-----------------------------------------------------------------------------
**	End of File
**-----------------------------------------------------------------------------
*/
#endif // __LIGHT__


