#ifndef RMSCENE_H
#define RMSCENE_H
/*
**-----------------------------------------------------------------------------
**  File:       RMScene.h
**  Purpose:    Sample RM Scene Management code
**  Notes:
**
**  Copyright (c) 1995 - 1997 by Microsoft, all rights reserved
**-----------------------------------------------------------------------------
*/

/*
**-----------------------------------------------------------------------------
**  Include files
**-----------------------------------------------------------------------------
*/
#include <vcl.h>
#pragma hdrstop

#include "Common.h"
#include "export.h"

#undef RELEASE
#ifdef __cplusplus
#define RELEASE(x) if (x != NULL) {x->Release(); x = NULL;}
#else
#define RELEASE(x) if (x != NULL) {x->lpVtbl->Release(x); x = NULL;}
#endif

/*
**-----------------------------------------------------------------------------
**  Classes
**-----------------------------------------------------------------------------
*/

/*
**-----------------------------------------------------------------------------
**  Name:	RMScene
**  Purpose:
**-----------------------------------------------------------------------------
*/

typedef BOOL CALLBACK (*CreateSceneCallbackType)(LPDIRECT3DRM3 lpD3DRM,LPDIRECT3DRMFRAME3 scene);

class PACKAGE RMScene : public I_Scene
{
public:
//------------interface-----------------
    virtual HRESULT STDMETHODCALLTYPE I_Init(LPI_D3DWindow lpd3dWindow);
    virtual HRESULT STDMETHODCALLTYPE I_Fini(void);
    virtual HRESULT STDMETHODCALLTYPE I_Attach(void);
    virtual HRESULT STDMETHODCALLTYPE I_Detach(void);
    virtual HRESULT STDMETHODCALLTYPE I_Render(void);
    virtual HRESULT STDMETHODCALLTYPE I_RenderSI(void);
    virtual HRESULT STDMETHODCALLTYPE I_SceneSetDraw2dCallback(BOOL (CALLBACK *fn)(HDC,D3DRECT,LPDDSURFACEDESC2,int));
    virtual HRESULT STDMETHODCALLTYPE I_SceneSetCommon2dCallback(bool CALLBACK (__closure *fn)(GraphInfo,D3DRECT,int));
    virtual HRESULT STDMETHODCALLTYPE I_GetSceneInfo(SceneInfo&);
//--------------------------------------
protected:
	// Data
    LPI_D3DWindow lpd3dWindow;		// Associated D3D Window interface
    LPDIRECT3DRMFRAME3 scene;       // scene frame
    LPDIRECT3DRMLIGHT ambient;      // scene ambient light
    BOOL (CALLBACK *Draw2dHandler)(HDC,D3DRECT,LPDDSURFACEDESC2,int);
//    BOOL (CALLBACK *Common2dHandler)(GraphInfo,D3DRECT,int);
    bool CALLBACK (__closure *Common2dHandler)(GraphInfo,D3DRECT,int);

public:

    RMScene (CreateSceneCallbackType fn=0);
    ~RMScene ();

    HRESULT Init (LPI_D3DWindow lpd3dWindow);
    HRESULT Fini (void);

    HRESULT Attach (void);
    HRESULT Detach (void);

    HRESULT AttachViewport (void);
    HRESULT DetachViewport (void);

    HRESULT Restore (void);

    BOOL CreateScene (void);

    HRESULT Render (void);
    HRESULT RenderSI(void);
    HRESULT Draw2d(GraphInfo grInfo,D3DRECT ClipRect,int type);

}; // End class RMScene

typedef RMScene* LPRMScene;

/*
**-----------------------------------------------------------------------------
**  End of File
**-----------------------------------------------------------------------------
*/
#endif // End RMSCENE_H
