#ifndef D3DWIN_H
#define D3DWIN_H

#include <mmsystem.h>
#include <vcl.h>
#pragma hdrstop

#include "Common.h"
#include "wn_DrvMgr.h"
#include "export.h"
#include "service3d.h"

// Windows messages
#define D3DWIN_GET_VALID    (WM_USER+1000)
#define D3DWIN_GET_POINTER  (WM_USER+1001)
#define D3DWIN_GET_SURFACE  (WM_USER+1002)

#define D3DWIN_INIT				  (WM_USER+2000)
#define D3DWIN_FINI				  (WM_USER+2001)
#define D3DWIN_CHANGED_DRIVER	  (WM_USER+2002)
#define D3DWIN_CHANGED_MODE		  (WM_USER+2003)
#define D3DWIN_CHANGED_DEVICE	  (WM_USER+2004)

#define D3DWIN_MIN_SIZE     64

// Forward declaration
class D3DWindow;
typedef D3DWindow * LPD3DWindow;

class PACKAGE D3DWindow : public I_D3DWindow
{
public:
//----------------interface-----------------------
  virtual HRESULT STDMETHODCALLTYPE I_AttachScene(LPI_Scene lpScene);
  virtual HRESULT STDMETHODCALLTYPE I_DetachScene(void);
  virtual void STDMETHODCALLTYPE I_GetGraphInfo(GraphInfo&);
  virtual BOOL STDMETHODCALLTYPE I_Draw(void);
  virtual HRESULT STDMETHODCALLTYPE I_CombineInterlacedFrame(void);

  virtual BOOL STDMETHODCALLTYPE I_isValid(void);
  virtual HRESULT STDMETHODCALLTYPE I_GetSurfaceRect(RECT&);
  virtual BOOL STDMETHODCALLTYPE I_NeedDXUpdateCall(void);

  virtual void STDMETHODCALLTYPE I_SetStereoBaseAndDistanse(float base,float distanse);
  virtual HRESULT STDMETHODCALLTYPE I_StartSI(void);
  virtual HRESULT STDMETHODCALLTYPE I_StopSI(void);

  virtual float STDMETHODCALLTYPE I_Pick(int x, int y,LPDIRECT3DRMFRAME3* lppick_frame);
//------------------------------------------------

protected:
//-----Flags-----
    enum {
	DWF_FULLSCREEN		= 0x00000001,
        DWF_VISIBLE  		= 0x00000002,
        DWF_ZBUFFER  		= 0x00000004,
	DWF_ACTIVE		= 0x00000008,
    } Attributes;

    enum {
        DWF_VALID_INTERFACE = 0x00000001,
	DWF_VALID_FULLSCREEN= 0x00000002,
	DWF_VALID_PRIMARY	= 0x00000004,
	DWF_VALID_RENDER    = 0x00000008,
	DWF_VALID_VIEWPORT  = 0x00000010,
	DWF_VALID_SCENE		= 0x00000020,
	DWF_VALID		    = 0x0000001F, // INTERFACE | FULLSCREEN | PRIMARY | RENDER | VIEWPORT
    } Validates;

public:
// Creation Methods
    D3DWindow (void);
    ~D3DWindow (void);
//    static LPDIRECT3DRM    lpD3DRM;     // Direct3DRM Interface
//    static LPDIRECT3DRM2   lpD3DRM2;    // Direct3DRM2 Interface

    HRESULT Create (
		HWND   hWnd,
		LPGUID lpDDGuid		= NULL,
		DWORD  dwW			= 0L,
		DWORD  dwH			= 0L,
		DWORD  dwBPP		= 0L,
		DWORD  dwRefresh	= 0L,
		LPGUID lpGuidD3D	= NULL,
		BOOL   fUseZBuffer	= TRUE);
    HRESULT Init (void);
    HRESULT Fini (void);

// Window Methods
    HRESULT DrawFrame (void);  //must call callback from scene object

    HRESULT Move (long x, long y);
    HRESULT Resize (DWORD w, DWORD h);

    HRESULT RealizePalette (void);
    HRESULT SetPalette ( int NumEntries, LPPALETTEENTRY lppe);

    HRESULT toGDI (void);
    HRESULT fromGDI (void);

    BOOL    isPaused (void)             { return (dwPaused != 0); }
    HRESULT Pause (BOOL fOn);

    BOOL    isFullscreen (void)			{ return ((fValid & DWF_VALID_FULLSCREEN) ? TRUE : FALSE); }

    HRESULT Restore (void );

	// Window State
    BOOL    isActive (void)				{ return ((fAttribs & DWF_ACTIVE) ? TRUE : FALSE); }
    void	turnActiveOn (void)			{ fAttribs |= DWF_ACTIVE; }
    void	turnActiveOff (void)		{ fAttribs &= ~DWF_ACTIVE; }

// Driver Methods
    HRESULT ChangeDesktop (void);
    HRESULT ChangeDriver (LPGUID lpGuid, LPD3DDevInfo lpDevHint = NULL,
						  LPDDModeInfo lpModeHint = NULL);
    HRESULT ChangeDevice (LPGUID lpGuid, LPDDModeInfo lpModeHint = NULL);
    HRESULT ChangeMode (DWORD w, DWORD h, DWORD bpp, DWORD refresh);
// Misc. Methods
    BOOL isValid (void)         { return (((fValid & DWF_VALID) == DWF_VALID) ? TRUE : FALSE); }

    BOOL isCreateZBuffer (void) { return (fAttribs & DWF_ZBUFFER); }
    void createZBufferOn (void) { fAttribs |= DWF_ZBUFFER; }
    void createZBufferOff (void){ fAttribs &= ~DWF_ZBUFFER; }
// Member methods
    HWND		  GetWindow (void)			{ return hWindow; }
    HCURSOR		  GetCursor (void)			{ return hCursorOld; }
    void		  SetCursor (HCURSOR hNew)	{ hCursorOld = hNew; }

    LPDDDrvInfo	  GetDriverInfo (void)	{ return lpCurrDriver; }
    LPDDModeInfo  GetModeInfo (void)	{ return lpCurrMode; }
    LPD3DDevInfo  GetDeviceInfo (void)	{ return lpCurrDevice; }

    LPGUID		  GetDDGuid (void)		{ return ((lpCurrDriver) ? lpCurrDriver->GetGuid () : NULL); }
    BOOL		  GetModeInfo (DWORD & w, DWORD & h,
							   DWORD & bpp, DWORD & refresh)
					{
						if (! lpCurrMode)
							return FALSE;

						lpCurrMode->GetMode (w, h , bpp, refresh);
						return TRUE;
					}
    LPGUID		  GetD3DGuid (void)		{ return ((lpCurrDevice) ? &(lpCurrDevice->guid) : NULL); }

    LPDIRECTDRAW  GetDD (void)			{ return lpDD; }
    LPDIRECTDRAW2 GetDD2 (void)			{ return lpDD2; }
    LPDIRECT3D2   GetD3D (void)			{ return lpD3D; }
  //  LPDIRECT3DRM  GetD3DRM (void) { return ::GetD3DRM(); }
//    LPDIRECT3DRM2 GetD3DRM2 (void) { return ::GetD3DRM2(); }
    LPDIRECT3DRMDEVICE2 GetD3DRM2Device (void) { return lpD3DRMDevice; }
    LPDIRECT3DRMFRAME2 GetCamera (void) 	      { return lpCamera; }
    LPDIRECT3DRMFRAME2 GetCameraLeft (void) 	  { return lpCameraLeft; }
    LPDIRECT3DRMFRAME2 GetCameraRight (void) 	  { return lpCameraRight; }
    LPDIRECT3DRMVIEWPORT GetD3DRMViewport (void)  { return lpD3DRMViewport; }

    LPDIRECTDRAWSURFACE GetPrimary (void)		{ return lpddsPrimary; }
    LPDIRECTDRAWSURFACE GetFrontBuffer (void)	{ return lpddsPrimary; }
    LPDIRECTDRAWPALETTE	GetPalette (void)		{ return lpddpPalette; }
    LPDIRECTDRAWCLIPPER GetClipper (void)       { return lpDDClipper; }

    LPDIRECTDRAWSURFACE GetRender (void)		{ return lpddsRender; }
    LPDIRECTDRAWSURFACE GetZBuffer (void)		{ return lpddsZBuff; }
    LPDIRECT3DDEVICE2   GetD3DDevice (void)		{ return lpd3dDevice; }
    LPDIRECT3DVIEWPORT2	GetViewport (void)		{ return lpd3dViewport; }

    HRESULT GetSurfaceRect (RECT & rSurface);
    HRESULT GetPrimaryRect (RECT & rPrimary);

//stereo
    float GetStereoBase() { return StereoBase; }
    float GetStereoDistanse() { return StereoDistanse; }
    void SetStereoBaseAndDistanse(float base,float distanse);
    HRESULT StartSI(void);
    HRESULT StopSI(void);
    HRESULT CombineInterlacedFrame();
    static void OpenStereoDriver(void);
    static void CloseStereoDriver(void);
//pick
    float Pick(int x, int y,LPDIRECT3DRMFRAME3* lppick_frame);

// Scene Methods
    LPI_Scene GetRMScene (void)					{ return lpRMScene; }

protected:
// Protected Methods
    HRESULT ValidateDefaults (void);

    HRESULT CreateInterfaces (LPGUID lpGuidDD);
    HRESULT InitInterfaces (void);
    HRESULT FiniInterfaces (void);

    HRESULT InitWindow (void);
    HRESULT FiniWindow (void);

    HRESULT InitFullscreenMode (void);
    HRESULT FiniFullscreenMode (void);

    HRESULT InitPrimary (void);
    HRESULT FiniPrimary (void);

    HRESULT InitPalette (void);

    HRESULT FiniPalette (void);

    HRESULT CreateRender (LPGUID lpD3DGuid);
public:
    HRESULT InitRender (void);
    HRESULT FiniRender (void);

    HRESULT InitViewport (void);
    HRESULT FiniViewport (void);

    HRESULT UpdateViewport (void);
protected:
	// Track initialization process
    void turnValidInterfaceOn (void)	{ fValid |= DWF_VALID_INTERFACE; }
    void turnValidInterfaceOff(void)	{ fValid &= ~DWF_VALID_INTERFACE; }

    void turnValidPrimaryOn (void)		{ fValid |= DWF_VALID_PRIMARY; }
    void turnValidPrimaryOff (void)		{ fValid &= ~DWF_VALID_PRIMARY; }

    void turnValidFullscreenOn (void)	{ fValid |= DWF_VALID_FULLSCREEN; }
    void turnValidFullscreenOff (void)	{ fValid &= ~DWF_VALID_FULLSCREEN; }

    void turnValidRenderOn (void)		{ fValid |= DWF_VALID_RENDER; }
    void turnValidRenderOff(void)		{ fValid &= ~DWF_VALID_RENDER; }

    void turnValidViewportOn (void)		{ fValid |= DWF_VALID_VIEWPORT; }
    void turnValidViewportOff(void)		{ fValid &= ~DWF_VALID_VIEWPORT; }

    void turnValidSceneOn (void)		{ fValid |= DWF_VALID_SCENE; }
    void turnValidSceneOff (void)		{ fValid &= ~DWF_VALID_SCENE;}

    // Flags
    BOOL isValidDefaults (void)		{ return ((lpCurrDriver && lpCurrMode && lpCurrDevice) ? TRUE : FALSE); }
    BOOL isValidInterface (void)	{ return ((fValid & DWF_VALID_INTERFACE) ? TRUE : FALSE); }
    BOOL isValidFullscreen (void)	{ return ((fValid & DWF_VALID_FULLSCREEN) ? TRUE : FALSE); }
    BOOL isValidPrimary (void)		{ return ((fValid & DWF_VALID_PRIMARY) ? TRUE : FALSE); }
    BOOL isValidRender (void)		{ return ((fValid & DWF_VALID_RENDER) ? TRUE : FALSE); }
    BOOL isValidViewport (void)		{ return ((fValid & DWF_VALID_VIEWPORT) ? TRUE : FALSE); }
    BOOL isValidScene (void)		{ return ((fValid & DWF_VALID_SCENE) ? TRUE : FALSE); }

    HRESULT CalcRects (void);

// Protected Data
    DWORD           dwSize;             // Structure Size
    DWORD           fAttribs;           // Attribute Flags
    DWORD           fChecks;            // Programming flags
    DWORD			fValid;				// Validation flags

// Window Data
    HWND            hWindow;            // Window handle
    DWORD           dwPaused;           // Paused Semaphore
    HCURSOR			hCursorOld;			// Old Cursor

    // Driver Defaults
    LPDDDrvInfo		lpCurrDriver;		// Current Driver
    LPDDModeInfo	lpCurrMode;			// Current Mode
    LPD3DDevInfo	lpCurrDevice;		// Current Device

	// Interfaces
    LPDIRECTDRAW    lpDD;               // DirectDraw Interface
    LPDIRECTDRAW2   lpDD2;              // DirectDraw2 Interface
    LPDIRECT3D2     lpD3D;              // Direct3D2 Interface
    LPDIRECT3DRMDEVICE2 lpD3DRMDevice;  //Direct3DRM2 device
    LPDIRECT3DRMFRAME2 lpCamera;           //Camera frame
    LPDIRECT3DRMFRAME2 lpCameraLeft;       //Camera frame Left
    LPDIRECT3DRMFRAME2 lpCameraRight;      //Camera frame Right
    LPDIRECT3DRMVIEWPORT lpD3DRMViewport;  //Direct3DRM2 Viewport
    LPDIRECT3DVIEWPORT lpd3dViewport1;     //need only for QIed IMM LPDIRECT3DVIEWPORT2 interface

    // Primary Surface
    LPDIRECTDRAWSURFACE  lpddsPrimary;  // Primary Surface
    LPDIRECTDRAWCLIPPER  lpDDClipper;   // Clipper object
    LPDIRECTDRAWPALETTE  lpddpPalette;  // Primary Palette
    DWORD				 cPalette;		// Count of palette entries
    PALETTEENTRY *       lppeSystem;    // Saved System palette entries
    PALETTEENTRY *       lppeCurr;		// Current palette entries

    // Render Surface
    LPDIRECTDRAWSURFACE  lpddsRender;   // Rendering surface
    LPDIRECTDRAWSURFACE  lpddsZBuff;    // Z-Buffer surface
    LPDIRECT3DDEVICE2    lpd3dDevice;	// D3D Device
    LPDIRECT3DVIEWPORT2  lpd3dViewport;	// D3D Viewport

    // Misc.
    RECT                rPrim;          // Current primary pos,size
    RECT                rSurf;          // Current surface pos,size

//stereo

    static  HDRVR     hSTEREO;
    static HANDLE hVxD;
    float StereoBase;
    float StereoDistanse;
//base viewport parameters(need to adjust viewport region for left and right and restore after SI)
    D3DVALUE d3dvLeft;
    D3DVALUE d3dvRight;
    D3DVALUE d3dvBottom;
    D3DVALUE d3dvTop;

    BOOL bNeedDXUpdateCall;

	//Scene
    LPI_Scene			lpRMScene;		// D3D RM Scene interface

}; // End D3DWindow

/*
**-----------------------------------------------------------------------------
**  End of File
**-----------------------------------------------------------------------------
*/
#endif // D3DWIN_H


