#ifndef RMWIN_H
#define RMWIN_H

#include <mmsystem.h>
#include <vcl.h>
#pragma hdrstop

#include "wn_debug.h"
#include "export.h"
#include "properties.h"

#undef RELEASE
#ifdef __cplusplus
#define RELEASE(x) if (x != NULL) {x->Release(); x = NULL;}
#else
#define RELEASE(x) if (x != NULL) {x->lpVtbl->Release(x); x = NULL;}
#endif

#define D3DCOLOR_ZBUF   D3DCOLOR_RGB+10
#define FLIPEVENT       "FlipSyncEvent"


class PACKAGE RMWindow : public I_D3DWindow
{
public:
//------------interface---------------
  virtual HRESULT STDMETHODCALLTYPE I_AttachScene(LPI_Scene lpScene);
  virtual HRESULT STDMETHODCALLTYPE I_DetachScene(void);
  virtual void STDMETHODCALLTYPE I_GetGraphInfo(GraphInfo&);
  virtual BOOL STDMETHODCALLTYPE I_Draw(void);
  virtual HRESULT STDMETHODCALLTYPE I_CombineInterlacedFrame(void);

  virtual BOOL STDMETHODCALLTYPE I_isValid(void);
  virtual HRESULT STDMETHODCALLTYPE I_GetSurfaceRect(RECT&);
  virtual BOOL STDMETHODCALLTYPE I_NeedDXUpdateCall(void);

  virtual void STDMETHODCALLTYPE I_SetStereoBaseAndDistanse(float base,float distanse);
  virtual HRESULT STDMETHODCALLTYPE I_StartSI(void);
  virtual HRESULT STDMETHODCALLTYPE I_StopSI(void);

  virtual float STDMETHODCALLTYPE I_Pick(int x, int y,LPDIRECT3DRMFRAME3* lppick_frame);
//-------------------------------------
    // Creation Methods
    __declspec(package) RMWindow(HWND hwnd, int w, int h, BOOL bSoftwareOnly=true, D3DCOLORMODEL model=D3DCOLOR_RGB, int bpp=0);
    __declspec(package) ~RMWindow();
//offscreen device creation function
    BOOL CreateOffScreenDevice( HWND hwnd, int w, int h, int flags=pt3D, D3DCOLORMODEL model=D3DCOLOR_RGB, int bpp=0);
    void ReleaseAllInterface(void);
    HRESULT InitImmRender(void);
    HRESULT InitImmRenderOffScreen(void);
    HRESULT FiniImmRender(void);
    HRESULT InitImmViewport(void);
    HRESULT FiniImmViewport(void);
    static DWORD bppToddbd(int bpp);
    static BOOL isPalettized (LPDDPIXELFORMAT lpddpf);
    static BOOL InitPalette(LPDIRECTDRAW4 lpDD2,LPDIRECTDRAWSURFACE4 surf);
    HRESULT SetPalette ( int NumEntries, LPPALETTEENTRY lppe);
    static LPGUID FindDevice(D3DCOLORMODEL cm, BOOL bSoftwareOnly);
    static BOOL RebuildDevice(HWND win, RMWindow* info, int width, int height);
    static BOOL ResizeViewport(HWND win, RMWindow* info, int width, int height);
//picking function
    float Pick(int x, int y,LPDIRECT3DRMFRAME3* lppick_frame);
// local service functions (macros)
    void  InitRMWindowConsts(void);
    bool  GetZBufferFormat(REFCLSID guid, int bpp, DDPIXELFORMAT *ddpfPixelFormat);

    BOOL isValid(void);
    LPDIRECTDRAW  GetDD (void)			{ return lpDD; }
    LPDIRECTDRAW4 GetDD2 (void)			{ return lpDD2; }
    LPDIRECT3D3   GetD3D (void)			{ return lpD3D; }
//    LPDIRECT3DRM  GetD3DRM (void)	   { return lpD3DRM; }
//    LPDIRECT3DRM3 GetD3DRM3 (void)	   { return lpD3DRM3; }
    LPDIRECT3DRMDEVICE3 GetD3DRM3Device (void)  { return lpD3DRMDevice; }
    LPDIRECT3DRMVIEWPORT2 GetD3DRMViewport(void){ return lpD3DRMViewport; }
    LPDIRECT3DRMFRAME3 GetCamera (void);
    LPDIRECT3DRMFRAME3 GetCameraLeft (void) 	{ return lpCameraLeft; }
    LPDIRECT3DRMFRAME3 GetCameraRight (void) 	{ return lpCameraRight; }
    LPDIRECT3DDEVICE3   GetD3DDevice (void)	{ return lpd3dDevice; }
    LPDIRECT3DVIEWPORT3	GetViewport (void)	{ return lpd3dViewport; }

    LPDIRECTDRAWSURFACE4 GetPrimary (void)	{ return lpddsPrimary; }
    LPDIRECTDRAWSURFACE4 GetRender (void)	{ return lpddsRender; }
    LPDIRECTDRAWSURFACE4 GetZBuffer (void)	{ return lpddsZBuff; }
    LPDIRECTDRAWCLIPPER GetClipper (void)       { return lpDDClipper; }

    HRESULT GetSurfaceRect (RECT & rSurface);
    LPI_Scene GetRMScene(void) { return lpRMScene; }

//stereo
    float GetStereoBase() { return StereoBase; }
    float GetStereoDistanse() { return StereoDistanse; }
    void SetStereoBaseAndDistanse(float base,float distanse);
    HRESULT StartSI(void);
    HRESULT StopSI(void);
    HRESULT CombineInterlacedFrame();
    static void OpenStereoDriver(void);
    static void CloseStereoDriver(void);

public:
    static RMWindow* active_window;
    BOOL bQuit;
    BOOL bMinimized;
    BOOL bFocus;
    LPI_Scene lpRMScene;
    HANDLE FlipEvnt;

//protected:
//    static LPDIRECT3DRM    lpD3DRM;     // Direct3DRM Interface
//    static LPDIRECT3DRM3   lpD3DRM3;    // Direct3DRM2 Interface

    LPDIRECTDRAW    lpDD;               // DirectDraw Interface
    LPDIRECTDRAW4   lpDD2;              // DirectDraw2 Interface
    LPDIRECT3D3     lpD3D;              // Direct3D2 Interface
    LPDIRECT3DRMDEVICE3 lpD3DRMDevice;  //Direct3DRM3 device
    LPDIRECT3DRMVIEWPORT2 lpD3DRMViewport;  //Direct3DRM2 Viewport
    LPDIRECT3DVIEWPORT lpd3dViewport1;     //need only for QIed IMM LPDIRECT3DVIEWPORT2 interface
    LPDIRECT3DDEVICE3    lpd3dDevice;	// D3D Device
    LPDIRECT3DVIEWPORT3  lpd3dViewport;	// D3D Viewport
    LPDIRECT3DRMFRAME3 lpCamera;           //Camera frame
    LPDIRECT3DRMFRAME3 lpCameraLeft;       //Camera frame Left
    LPDIRECT3DRMFRAME3 lpCameraRight;      //Camera frame Right

    // Primary Surface
    LPDIRECTDRAWSURFACE4  lpddsPrimary;                 // Primary (screen) Surface
    LPDIRECTDRAWSURFACE4  lpddsRender;                  // Rendering surface
//    LPDIRECTDRAWSURFACE4  lpddsLeft,lpddsRight;         // Stereo surfaces
//    LPDIRECTDRAWSURFACE4  lpddsLeftBack,lpddsRightBack; // Stereo buffers
    LPDIRECTDRAWSURFACE4  lpddsZBuff;                   // Z-Buffer surface
    LPDIRECTDRAWCLIPPER  lpDDClipper;                   // Clipper object

    D3DRMCOLORMODEL model;
    BOOL bNeedDXUpdateCall;

//stereo
    static HDRVR hSTEREO;
    static HANDLE hVxD;
    BOOL bStereoInterlaced;
    float StereoBase;
    float StereoDistanse;
    void __fastcall ClearStereoImages();
    bool __fastcall BltStereoImages(HDC LeftDC, RECT *LeftSrcRect, RECT *LeftDesRECT,
                                         HDC RightDC, RECT *RightSrcRect, RECT *RightDesRECT);
    bool __fastcall BltImage(HDC SrcDC,LPDIRECTDRAWSURFACE4 DDSurf,RECT *SrcRect,RECT *DesRECT);
//base viewport parameters(need to adjust viewport region for left and right and restore after SI)
    D3DVALUE d3dvLeft;
    D3DVALUE d3dvRight;
    D3DVALUE d3dvBottom;
    D3DVALUE d3dvTop;
}; // End RMWindow

#endif // RMWIN_H

