//---------------------------------------------------------------------------
#ifndef propertiesH
#define propertiesH

#include <d3drm.h>
#pragma hdrstop

#include "camera.h"
#include "stctrl.h"
#include "flipthread.h"

class TStereoControl;
class TD3Light;

enum DRV          { drSoft, drHard};
enum FILL         { flPoint, flWireFrame, flFlat, flShaded }; // need for VCL compatability see SetFill() before change
enum CHANGEVECTOR { cvNone, cvX, cvY, cvZ, cvAll };
enum PANELTYPE    { pt3D=1, ptStereo=2, ptStereo3D=3};
enum CLEARFLG     { clFront=1, clBack=2};
const int ptSoft=4;
const int BUF_DELTA=10;
//---------------------------------------------------------------------------
class PACKAGE TD3Vector: public TPersistent
{
  D3DVECTOR FVect;
  void __fastcall SetX(float value);
  void __fastcall SetY(float value);
  void __fastcall SetZ(float value);
  void __fastcall SetVector(D3DVECTOR value);
public:
  TD3Vector(float x=0., float y=0., float z=0.);
  TD3Vector& operator *= (const TD3Vector& v);
//  virtual void __fastcall Assign(TD3Vector *);
  __property D3DVECTOR Vector = { read = FVect, write = SetVector };
  void __fastcall (__closure *OnChange)(TD3Vector *, CHANGEVECTOR comp=cvAll);
__published:
  __property float x  = { read = FVect.x, write = SetX };
  __property float y  = { read = FVect.y, write = SetY };
  __property float z  = { read = FVect.z, write = SetZ };
};
//---------------------------------------------------------------------------
class PACKAGE TD3CommonLight: public TPersistent
{
  TColor FAmbient, FDirectional;
  TD3Vector *FDirection;
  TStereoControl *D3Panel;
  void __fastcall SetDirection(TD3Vector *value);
  void __fastcall SetAmbient(TColor value);
  void __fastcall SetDirectional(TColor value);
public:
  TD3CommonLight( TStereoControl *panel);
  __fastcall ~TD3CommonLight();
  void __fastcall Refresh(void);
__published:
  __property TColor Ambient  = { read = FAmbient, write = SetAmbient };
  __property TColor Directional  = { read = FDirectional, write = SetDirectional };
  __property TD3Vector *Direction  = { read = FDirection, write = SetDirection };
};
//---------------------------------------------------------------------------
class PACKAGE TD3CameraProp: public TPersistent
{
  TStereoControl *D3Panel;
  float FLens;
  float FFront;
  float FBack;
  bool  FShowSky, FShowGround;
  TD3LODs * FDetailLevels;
  bool FDetailEnabled;
  TD3Vector *FDirection, *FUp, *FPosition, *FRotation, *FVelocity;
  void __fastcall SetPosition(TD3Vector *value);
  void __fastcall SetDirection(TD3Vector *value);
  void __fastcall SetUp(TD3Vector *value);
  void __fastcall SetVelocity(TD3Vector *value);
  void __fastcall SetRotation(TD3Vector *value);
  void __fastcall SetLens(float value);
  void __fastcall SetFront(float value);
  void __fastcall SetBack(float value);
  void __fastcall SetShowSky(bool value);
  bool __fastcall GetShowSky();
  void __fastcall SetShowGround(bool value);
  bool __fastcall GetShowGround();
  void __fastcall SetDetailEnabled(bool value);
  bool __fastcall GetDetailEnabled();
  TD3LODs* __fastcall GetDetailLevels();
protected:
  virtual void __fastcall OnChangeVector(TD3Vector *vect, CHANGEVECTOR comp=cvAll);
public:
  TD3CameraProp( TStereoControl *panel);
  __fastcall ~TD3CameraProp();
  void __fastcall Refresh(void);
  __property TD3LODs * DetailLevels  = { read=GetDetailLevels };
__published:
  __property TD3Vector *Position  = { read = FPosition, write = SetPosition };
  __property TD3Vector *Velocity  = { read = FVelocity, write = SetVelocity };
  __property TD3Vector *Rotation  = { read = FRotation, write = SetRotation };
  __property TD3Vector *Direction  = { read = FDirection, write = SetDirection };
  __property TD3Vector *Up  = { read = FUp, write = SetUp };
  __property float Lens  = { read = FLens, write = SetLens };
  __property float Front  = { read = FFront, write = SetFront };
  __property float Back  = { read = FBack, write = SetBack };
  __property bool  ShowSky  = { read=GetShowSky, write=SetShowSky };
  __property bool  ShowGround  = { read=GetShowGround, write=SetShowGround };
  __property bool DetailEnabled  = { read=GetDetailEnabled, write=SetDetailEnabled };
};
//---------------------------------------------------------------------------
class PACKAGE TD3Fog: public TPersistent
{
    bool FEnabled;
    TColor FColor;
    D3DVALUE FFront, FBack;
    TStereoControl *D3Panel;
    void __fastcall SetEnabled(bool value);
    void __fastcall SetFront(D3DVALUE value);
    void __fastcall SetBack(D3DVALUE value);
    void __fastcall SetColor(TColor value);
public:
    __fastcall TD3Fog( TStereoControl *panel);
    void __fastcall Refresh(void);
__published:
    __property bool Enabled  = { read = FEnabled, write = SetEnabled };
    __property TColor Color  = { read = FColor, write = SetColor };
    __property D3DVALUE Front  = { read = FFront, write = SetFront };
    __property D3DVALUE Back  = { read = FBack, write = SetBack };
};
//---------------------------------------------------------------------------
class PACKAGE TD3Engine: public TPersistent
{
private:
    TStereoControl   *D3Panel;
    DRV        FDriver;
    bool       FActive, FEnabled, FDither;
	 FILL       FFill;
    D3DRMTEXTUREQUALITY FTextureFiltering;
	 void __fastcall SetDither(bool value);
	 bool __fastcall GetDither();
	 void __fastcall SetTextureFiltering(D3DRMTEXTUREQUALITY value);
	 D3DRMTEXTUREQUALITY __fastcall GetTextureFiltering();
	 void __fastcall SetDriver(DRV value);
	 void __fastcall SetActive(bool value);
	 void __fastcall SetEnabled(bool value);
	 void __fastcall SetFill(FILL value);
	 void __fastcall SetStereoActive(bool value);
	 bool __fastcall GetStereoActive();
public:
	 TD3Engine( TStereoControl *panel);
	 void __fastcall Refresh(void);
	 __property bool Active  = { read = FActive, write = SetActive };
	__property bool StereoActive  = { read=GetStereoActive, write=SetStereoActive };
__published:
	__property bool Enabled  = { read = FEnabled, write = SetEnabled };
	__property DRV Driver  = { read = FDriver, write = SetDriver };
	__property FILL Fill  = { read = FFill, write = SetFill };
	__property bool Dither  = { read=GetDither, write=SetDither };
	__property D3DRMTEXTUREQUALITY TextureFiltering  = { read=GetTextureFiltering, write=SetTextureFiltering };
};
//---------------------------------------------------------------------------
class PACKAGE TGlasses;
class PACKAGE TStereo: public TPersistent
{
private:
	TStereoControl   *D3Panel;
	bool             FActive,  FlipLeft, FlipRight, FSaveSizeY, FSaveSizeX, IsOverlay, UseOverlay, FReversed;
	HANDLE           FlipEvent;
	int              FWidth, FHeight, BufWidth, BufHeight;
	int              FMode, CurrMode, FFrequency, FlipTick, FlipCount;
	int				  FMonoPage;
	int              CursorHotX, CursorHotY, MouseX, MouseY;
	int              FMouseDx, FMouseDy, FEyesDx, LeftRightDx;
	int              FLeftDx, FLeftDy, FRightDx, FRightDy;
	TColor           FCursorColor;
	TColor           FColorLeft, FColorRight;
	Graphics::TBitmap *LeftCursor, *RightCursor, *OriginCursor;
	Graphics::TBitmap *bmpl, *bmpr, *icn;	//   (   () )
	TFlipThread      *flipthread;
	HANDLE           Events[2];   // page flip events
	LPDIRECTDRAWSURFACE4  SurfLeft, SurfRight, SurfLeftBack, SurfRightBack, SurfCursor; // Real Stereo buffers
	LPDIRECTDRAWSURFACE4  CurrLeft, CurrRight, CurrLeftBack, CurrRightBack; 				// Logical Stereo buffers
	RECT				  LeftSrcRect, RightSrcRect, LeftBufRect, RightBufRect;

	void __fastcall  SetActive   (bool value);
	void __fastcall  SetHeight   (int ht);
	void __fastcall  SetWidth    (int wd);
	void __fastcall  SetMode     (int mode);
	int  __fastcall  GetFillMask (LPDIRECTDRAWSURFACE4 surf, TColor color);
	void __fastcall  SetMouseDx  (int dx);
	void __fastcall  SetMouseDy  (int dy);
	void __fastcall  SetLeftDx   (int dx);
	void __fastcall  SetLeftDy   (int dy);
	void __fastcall  SetRightDx  (int dx);
	void __fastcall  SetRightDy  (int dy);
	void __fastcall  SetCursorColor(TColor Value);
	void __fastcall  SetColorRight(TColor value);
	void __fastcall  SetColorLeft(TColor value);
	void __fastcall  DeleteBuffs ();
	bool __fastcall  CreateBuffs ();
	void __fastcall  UpdateCursor();
	void __fastcall  UpdateOverlay(RECT &src, RECT &dest);
	void __fastcall  SetMonoPage(int value);
	void __fastcall  SetEyesDx(int value);
	bool __fastcall  SetBufPos(RECT &bmp, RECT &src, RECT &buf);
public:
	TStereo(TStereoControl *panel);
	__fastcall ~TStereo();

	virtual    void     __fastcall Paint     (void);
	virtual    void     __fastcall Flip      (int time=0);
	virtual    void     __fastcall Flip0     (int time=-1);
	virtual    bool     __fastcall Update    (RECT *rect=NULL);
	virtual    void     __fastcall Clear     (int pair=-1, int flg=clFront|clBack);
	virtual    void     __fastcall Clear     (int pair, int flg, RECT *upd);
	virtual	  void  	  __fastcall Clear	  (LPDIRECTDRAWSURFACE4  Surf, RECT *rct);
	virtual    void     __fastcall DrawCursor(int X, int Y, bool redraw=false);
	virtual    void     __fastcall SetCursorBitmap(HBITMAP bmp, int hotx=0, int hoty=0);
	virtual    void     __fastcall SetCursor (HICON *cursor=NULL);
	virtual    void     __fastcall SetSaveSizeX  (bool value);
	virtual    void     __fastcall SetSaveSizeY  (bool value);
	virtual    void     __fastcall SetReversed   (bool value);

	TStringList           *Modes;
	Graphics::TBitmap     *LeftImg, *RightImg, *StereoImg;	//   (    )
	bool		  UpdateBuf;	

	__property bool     Active   = { read = FActive, 	write = SetActive };
	__property int      MouseDx  = { read = FMouseDx, 	write = SetMouseDx };
	__property int      MouseDy  = { read = FMouseDy, 	write = SetMouseDy };
	__property int      LeftDx   = { read = FLeftDx,  	write = SetLeftDx };
	__property int      LeftDy   = { read = FLeftDy,  	write = SetLeftDy };
	__property int      RightDx  = { read = FRightDx, 	write = SetRightDx };
	__property int      RightDy  = { read = FRightDy, 	write = SetRightDy };

__published:
	__property int      Width    = { read = FWidth,   write = SetWidth };
	__property int      Height   = { read = FHeight,  write = SetHeight };
	__property int      Mode     = { read = FMode,    write = SetMode };
	__property int      Frequency= { read = FFrequency };
	__property TColor   CursorColor= { read = FCursorColor,write = SetCursorColor};
	__property TColor   ColorLeft= { read = FColorLeft,write = SetColorLeft };
	__property TColor   ColorRight={ read = FColorRight,write = SetColorRight };
	__property bool     SaveSizeX= { read = FSaveSizeX, write = SetSaveSizeX };
	__property bool     SaveSizeY= { read = FSaveSizeY, write = SetSaveSizeY };
	__property bool     Reversed = { read = FReversed,  write = SetReversed };
	__property int		  MonoPage = { read = FMonoPage,  write = SetMonoPage };
	__property int		  EyesDx	  = { read = FEyesDx,	 write = SetEyesDx };
};
//---------------------------------------------------------------------------
class PACKAGE TGlasses: public TPersistent
{
	virtual void   __fastcall     SetActive   (bool value)   {};
	virtual bool   __fastcall     GetActive   (void)         {RefreshInfo(); return Config.Active;};
	virtual void   __fastcall     SetReversed (bool value)   {};
	virtual bool   __fastcall     GetReversed (void)         {RefreshInfo(); return Config.Reversed;};
	virtual void   __fastcall     SetInfo     (StereoConfig *Config) {};
   virtual StereoConfig* __fastcall GetInfo  (void)         {RefreshInfo(); return &Config;};
   virtual TStringList* __fastcall GetSupportedModes(void)     {RefreshInfo(); return Modes;};
   virtual int    __fastcall     GetMode     (void)         {RefreshInfo(); return Config.Mode;};
   virtual void   __fastcall     SetMode     (int mode)     {};
   virtual float  __fastcall     GetDriverVersion (void)    {return Config.DriverVersion;};
   virtual int    __fastcall     GetPort     (void)         {return Config.Port;};
   virtual int    __fastcall     GetPortNum  (void)         {RefreshInfo(); return Config.PortNum;};
   virtual void   __fastcall     SetPortNum  (int port)     {};
   virtual bool   __fastcall     RefreshInfo (void)         {return false;};

protected:
	StereoConfig   Config;
	char     		*ModeName[smMono+1];
    static AnsiString  ConfigRegPath;
	HWND            WndOfDriver;
	bool   	    	UnloadOnExit;
public:
	TGlasses();
	__fastcall ~TGlasses();

	int         Type;
	AnsiString  Name, Chipset;
	TStringList *Modes;

	static  int  __fastcall GetConnectedGlasses (HWND *wnd=NULL, bool *load_now=NULL);
	virtual void __fastcall Exit(void) {};
	virtual int  __fastcall SetCompatibleMode   (int mode) {return smDefault;};
            void __fastcall SendMsg2Driver(COPYDATASTRUCT *cds);

	__property  TStringList *SupportedModes =  { read = GetSupportedModes };
	__property  float    DriverVersion =  { read = GetDriverVersion };
	__property  StereoConfig *Info= { read = GetInfo, write = SetInfo };
	__property  bool     Active  =  { read = GetActive, write = SetActive };
	__property  bool     Reversed=  { read = GetReversed,write = SetReversed };
	__property  int      Mode    =  { read = GetMode,   write = SetMode   };
	__property  int      Port    =  { read = GetPort };
	__property  int      PortNum =  { read = GetPortNum,write = SetPortNum };
};
//---------------------------------------------------------------------------
class PACKAGE TGlassesIBIK: public TGlasses
{
	virtual void   __fastcall     SetActive   (bool value);
	virtual void   __fastcall     SetInfo     (StereoConfig *Config);
	virtual void   __fastcall     SetMode     (int mode);
	virtual void   __fastcall     SetPortNum  (int port);
	virtual bool   __fastcall     RefreshInfo (void);
	virtual void   __fastcall     SetReversed (bool value);
public:
	TGlassesIBIK();
	__fastcall ~TGlassesIBIK();
	virtual void   __fastcall     Exit        (void);
	virtual int    __fastcall     SetCompatibleMode(int mode);
};
//---------------------------------------------------------------------------
#endif

