#ifndef __MATERIAL__
#define __MATERIAL__
/*
**-----------------------------------------------------------------------------
** Name:    D3DUtils.h
** Purpose: Various D3D IMM material functions
** Notes:
**
** Copyright (c) 1995 - 1997 by Microsoft, all rights reserved
**-----------------------------------------------------------------------------
*/

#define D3D_OVERLOADS
#include <vcl.h>
#include <d3d.h>
#include <d3dtypes.h>
#include <list>
#pragma hdrstop

#include "export.h"
#include "sc_Tex.h"
/*
**-----------------------------------------------------------------------------
** Classes
**-----------------------------------------------------------------------------
*/

/*
**-----------------------------------------------------------------------------
**	Class:		Material
**	Purpose:
**-----------------------------------------------------------------------------
*/

class Material;  //forward declaration
typedef Material* LPMaterial;
typedef std::list<LPMaterial> LIST_Material;  //std::  instead of (using namespace std;)

class PACKAGE Material
{
protected:
	static LIST_Material list;
	D3DMATERIAL	    Mat;
        D3DMATERIALHANDLE   hMat;
	LPDIRECT3DMATERIAL3 lpMat;
	int	            changed;
	LPD3DTexture        clpD3DTexture;
public:
	Material();
	virtual ~Material();

	HRESULT		SetAsCurrent(LPDIRECT3DDEVICE3 lpDev);
	HRESULT		SetAsBackground(LPDIRECT3DVIEWPORT2 lpView);
	HRESULT		Set(void);

	void		SetDiffuse(const D3DVECTOR& color);
	D3DVECTOR	GetDiffuse(void) const;
	void		SetAlpha(const float& alpha);
	float		GetAlpha(void) const;
	void		SetAmbient(const D3DVECTOR& color);
	D3DVECTOR	GetAmbient(void) const;
	void		SetEmissive(const D3DVECTOR& color);
	D3DVECTOR	GetEmissive(void) const;
	void		SetSpecular(const D3DVECTOR& color);
	D3DVECTOR	GetSpecular(void) const;
	void		SetPower(const D3DVALUE& power);
	D3DVALUE	GetPower(void) const;
	void		SetRampSize(const DWORD& ramp);
	DWORD		GetRampSize(void) const;

	void CreateMaterial(LPDIRECT3D3 lpD3D, LPDIRECT3DDEVICE3 lpDev);
	void SetTexture(LPD3DTexture lpD3DTexture);
	void SetTextureHandle(const D3DTEXTUREHANDLE& hTexture);
	LPDIRECT3DMATERIAL3 GetMaterial(void) { return lpMat; }
	void SetHandle(D3DMATERIALHANDLE	h) {hMat=h;}

	static LIST_Material& GetList() { return list; }
	static void AfterDeviceCreated(GraphInfo&);
        static void BeforeDeviceDestroyed(GraphInfo&);
        static BOOL AdjustHandleForDevice(IDirect3DDevice3 *Device);

protected:
	D3DTEXTUREHANDLE GetTextureHandle(void);
        void RestoreAfterDeviceCreated(GraphInfo&);
	void Release(void);

}; // End Material


/*
**-----------------------------------------------------------------------------
**	End of File
**-----------------------------------------------------------------------------
*/
#endif // __MATERIAL__


